<?php

if ( !defined( 'ABSPATH' ) )
	exit;

/**
 * Class to register all settings in the settings panel
 */
class fdmSettings {

	/**
	 * Default values for settings
	 * @since 2.0.0
	 */
	public $defaults = array();

	/**
	 * Stores the legacy premium settings
	 * @since 2.0.0
	 */
	public $premium_settings = array();

	/**
	 * Stored values for settings
	 * @since 2.0.0
	 */
	public $settings = array();

	public function __construct() {

		add_action( 'init', array( $this, 'set_defaults' ) );

		// Call when plugin is initialized on every page load
		add_action( 'init', array( $this, 'load_settings_panel' ) );

		// Add filters on the menu style so we can apply the setting option
		add_filter( 'fdm_menu_args', array( $this, 'set_style' ) );
		add_filter( 'fdm_shortcode_menu_atts', array( $this, 'set_style' ) );
		add_filter( 'fdm_shortcode_menu_item_atts', array( $this, 'set_style' ) );

	}

	/**
	 * Set the default values for different settings in the plugin
	 *
	 * @since 1.5
	 */
	public function set_defaults() {

		$this->defaults = array(
			'fdm-pro-style' => 'classic',
			'fdm-sidebar-click-action' => 'onlyselected',
			'fdm-menu-section-image-placement' => 'hidden',
			'fdm-related-items' => 'none',
			'fdm-currency-symbol-location' => 'before',
			'fdm-image-style-columns' => 'four',
			'fdm-item-flag-icon-size' => '32',
			'fdm-price-filtering-type' => 'textbox',
		);

		$this->defaults = apply_filters( 'fdm_defaults', $this->defaults );
	}

	/**
	 * Get the theme supports options for this plugin
	 *
	 * This mimics the core get_theme_support function, except it automatically
	 * looks up this plugin's feature set and searches for features within
	 * those settings.
	 *
	 * @param string $feature The feature support to check
	 * @since 1.5
	 */
	public function get_theme_support( $feature ) {

		$theme_support = get_theme_support( 'food-and-drink-menu' );

		if ( !is_array( $theme_support ) ) {
			return apply_filters( 'fdm_get_theme_support_' . $feature, false, $theme_support );
		}

		$theme_support = $theme_support[0];

		if ( isset( $theme_support[$feature] ) ) {
			return apply_filters( 'fdm_get_theme_support_' . $feature, $theme_support[$feature], $theme_support );
		}

		return apply_filters( 'fdm_get_theme_support_' . $feature, false, $theme_support );
	}

	/**
	 * Get a setting's value or fallback to a default if one exists
	 * @since 2.0.0
	 */
	public function get_setting( $setting ) {

		if ( empty( $this->settings ) ) {
			$this->settings = get_option( 'food-and-drink-menu-settings' );
		}

		if ( !empty( $this->settings[ $setting ] ) ) {
			return apply_filters( 'fdm-setting-' . $setting, $this->settings[ $setting ] );
		}

		if ( empty( $this->premium_settings ) ) {
			$this->premium_settings = get_option( 'food-and-drink-menu-extra-settings' );
		}

		if ( !empty( $this->premium_settings[ $setting ] ) ) {
			return apply_filters( 'fdm-setting-' . $setting, $this->premium_settings[ $setting ] );
		}

		if ( !empty( $this->defaults[ $setting ] ) ) {
			return apply_filters( 'fdm-setting-' . $setting, $this->defaults[ $setting ] );
		}

		return apply_filters( 'fdm-setting-' . $setting, null );
	}

	/**
	 * Load the admin settings page
	 * @since 1.1
	 * @sa https://github.com/NateWr/simple-admin-pages
	 */
	public function load_settings_panel() {
		global $fdm_controller;

		require_once( FDM_PLUGIN_DIR . '/lib/simple-admin-pages/simple-admin-pages.php');

		// Insantiate the Simple Admin Library so that we can add a settings page
		$sap = sap_initialize_library(
			array(
				'version'		=> '2.0.a.7', // Version of the library
				'lib_url'		=> FDM_PLUGIN_URL . '/lib/simple-admin-pages/', // URL path to sap library
			)
		);

		// Create a page for the options under the Settings (options) menu
		$sap->add_page(
			'submenu', 				// Admin menu which this page should be added to
			array(					// Array of key/value pairs matching the AdminPage class constructor variables
				'id'			=> 'food-and-drink-menu-settings',
				'title'			=> __( 'Settings', 'food-and-drink-menu' ),
				'menu_title'	=> __( 'Settings', 'food-and-drink-menu' ),
				'description'	=> '',
				'capability'	=> 'manage_options',
				'parent_menu'	=> 'edit.php?post_type=fdm-menu',
				'default_tab'	=> 'fdm-basic-settings'
			)
		);

		// Create a tab for basic settings
		$sap->add_section(
			'food-and-drink-menu-settings',	// Page to add this section to
			array(								// Array of key/value pairs matching the AdminPageSection class constructor variables
				'id'			=> 'fdm-basic-settings',
				'title'			=> __( 'Basic', 'food-and-drink-menu' ),
				'is_tab'		=> true
			)
		);

		// Create a section to choose a default style
		$sap->add_section(
			'food-and-drink-menu-settings',	// Page to add this section to
			array(								// Array of key/value pairs matching the AdminPageSection class constructor variables
				'id'			=> 'fdm-style-settings',
				'title'			=> __( 'Style', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose what style you would like to use for your menu.', 'food-and-drink-menu' ),
				'tab'			=> 'fdm-basic-settings'
			)
		);

		$options = array();
		foreach( $fdm_controller->styles as $style ) {
			$options[$style->id] = $style->label;
		}
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-style-settings',
			'select',
			array(
				'id'			=> 'fdm-style',
				'title'			=> __( 'Menu Formatting', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the formatting for your menus.', 'food-and-drink-menu' ),
				'blank_option'	=> false,
				'options'		=> $options
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-style-settings',
			'toggle',
			array(
				'id'			=> 'fdm-sidebar',
				'title'			=> __( 'Enable Sidebar', 'food-and-drink-menu' ),
				'label'			=> __( 'Display a sidebar for your menu that allows visitors to choose what section they want to view.', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-style-settings',
			'select',
			array(
				'id'			=> 'fdm-sidebar-click-action',
				'title'			=> __( 'Sidebar Click Action', 'food-and-drink-menu' ),
				'description'	=> __( '(Only applicable if the sidebar is enabled.) Choose what happens when you click a section in the menu sidebar. Only Selected will display only the chosen section, with no scrolling feature. Scroll displays all the sections and then scrolls to the chosen one.', 'food-and-drink-menu' ),
				'blank_option'	=> false,
				'options'		=> array(
					'onlyselected' 	=> 'Only Selected',
					'scroll' 	=> 'Scroll',
				)
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-style-settings',
			'select',
			array(
				'id'			=> 'fdm-menu-section-image-placement',
				'title'			=> __( 'Section Images', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the location, if any, for your section images relative to the title.', 'food-and-drink-menu' ),
				'blank_option'	=> false,
				'options'		=> array(
					'hidden' 		=> 'Hidden',
					'background' 	=> 'Background',
					'above' 		=> 'Above',
					'below' 		=> 'Below'
				)
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-style-settings',
			'toggle',
			array(
				'id'			=> 'fdm-display-section-descriptions',
				'title'			=> __( 'Display Section Descriptions', 'food-and-drink-menu' ),
				'label'			=> __( 'Enable this if you want to display the section descriptions in the main menu area.', 'food-and-drink-menu' ),
			)
		);

		// Create a section to enable/disable specific features
		$sap->add_section(
			'food-and-drink-menu-settings',	// Page to add this section to
			array(								// Array of key/value pairs matching the AdminPageSection class constructor variables
				'id'			=> 'fdm-enable-settings',
				'title'			=> __( 'Functionality', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose what features of the menu items you wish to enable or disable.', 'food-and-drink-menu' ),
				'tab'			=> 'fdm-basic-settings'
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-enable-settings',
			'toggle',
			array(
				'id'			=> 'fdm-disable-price',
				'title'			=> __( 'Disable Price', 'food-and-drink-menu' ),
				'label'			=> __( 'Disable all pricing options for menu items.', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-enable-settings',
			'toggle',
			array(
				'id'			=> 'fdm-details-lightbox',
				'title'			=> __( 'Details Lightbox', 'food-and-drink-menu' ),
				'label'			=> __( 'Should visitors be able to click on menu items to view more details about them (custom fields, related items, etc.).', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-enable-settings',
			'text',
			array(
				'id'			=> 'fdm-currency-symbol',
				'title'			=> __( 'Currency Symbol', 'food-and-drink-menu' ),
				'label'			=> __( 'The symbol added either before or after menu prices.', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-enable-settings',
			'select',
			array(
				'id'			=> 'fdm-currency-symbol-location',
				'title'			=> __( 'Currency Symbol Location', 'food-and-drink-menu' ),
				'label'			=> __( 'Choose whether the currency symbol be displayed before or after the price.', 'food-and-drink-menu' ),
				'blank_option'	=> false,
				'options'		=> array(
					'before' 	=> 'Before',
					'after' 	=> 'After'
				)
			)
		);

		// Create a section for advanced options
		// $sap->add_section(
		// 	'food-and-drink-menu-settings',	// Page to add this section to
		// 	array(								// Array of key/value pairs matching the AdminPageSection class constructor variables
		// 		'id'			=> 'fdm-advanced-settings',
		// 		'title'			=> __( 'Advanced Options', 'food-and-drink-menu' )
		// 	)
		// );
		// $sap->add_setting(
		// 	'food-and-drink-menu-settings',
		// 	'fdm-advanced-settings',
		// 	'text',
		// 	array(
		// 		'id'			=> 'fdm-item-thumb-width',
		// 		'title'			=> __( 'Menu Item Photo Width', 'food-and-drink-menu' ),
		// 		'description'	=> sprintf(
		// 			esc_html__( 'The width in pixels of menu item thumbnails. Leave this field empty to preserve the default (600x600). After changing this setting, you may need to %sregenerate your thumbnails%s.', 'food-and-drink-menu' ),
		// 			'<a href="http://doc.themeofthecrop.com/plugins/food-and-drink-menu/user/faq#image-sizes">',
		// 			'</a>'
		// 		),
		// 	)
		// );
		// $sap->add_setting(
		// 	'food-and-drink-menu-settings',
		// 	'fdm-advanced-settings',
		// 	'text',
		// 	array(
		// 		'id'			=> 'fdm-item-thumb-height',
		// 		'title'			=> __( 'Menu Item Photo Height', 'food-and-drink-menu' ),
		// 		'description'	=> sprintf(
		// 			esc_html__( 'The height in pixels of menu item thumbnails. Leave this field empty to preserve the default (600x600). After changing this setting, you may need to %sregenerate your thumbnails%s.', 'food-and-drink-menu' ),
		// 			'<a href="http://doc.themeofthecrop.com/plugins/food-and-drink-menu/user/faq#image-sizes">',
		// 			'</a>'
		// 		),
		// 	)
		// );

		if ( ! $fdm_controller->permissions->check_permission('styling') ) {
			$styling_permissions = array(
				'disabled'		=> true,
				'disabled_image'=> 'https://www.etoilewebdesign.com/wp-content/uploads/2018/06/Logo-White-Filled40-px.png',
				'purchase_link'	=> 'https://www.etoilewebdesign.com/plugins/ultimate-product-catalog/'
			);
		}
		else { $styling_permissions = array(); }

		// Create a tab for styling settings
		$sap->add_section(
			'food-and-drink-menu-settings',	// Page to add this section to
			array(								// Array of key/value pairs matching the AdminPageSection class constructor variables
				'id'			=> 'fdm-styling-settings-tab',
				'title'			=> __( 'Styling', 'food-and-drink-menu' ),
				'is_tab'		=> true
			)
		);
		// Create a section for styling options
		$sap->add_section(
			'food-and-drink-menu-settings',	// Page to add this section to
			array_merge( 
				array(								// Array of key/value pairs matching the AdminPageSection class constructor variables
					'id'			=> 'fdm-styling-settings',
					'title'			=> __( 'Styling Options', 'food-and-drink-menu' ),
					'description'	=> __( 'Choose what filtering, if any, of the menu items you wish to enable.', 'food-and-drink-menu' ),
					'tab'			=> 'fdm-styling-settings-tab'
				),
				$styling_permissions
			)
		);
		$style_options = array();
		foreach( $fdm_controller->prostyles as $prostyle ) {
			$style_options[$prostyle->id] = $prostyle->label;
		}
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'select',
			array(
				'id'			=> 'fdm-pro-style',
				'title'			=> __( 'Menu Style', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the style for your menus.', 'food-and-drink-menu' ),
				'blank_option'	=> false,
				'options'		=> $style_options
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'radio',
			array(
				'id'			=> 'fdm-image-style-columns',
				'title'			=> __( 'Image Style Columns', 'food-and-drink-menu' ),
				'description'	=> __( '(Only applicable if Image Style is selected above.) Choose how many columns you want to display in the Image Style layout.', 'food-and-drink-menu' ),
				'options'		=> array(
					'three'		=> '3',
					'four'		=> '4',
					'five'		=> '5'
				)
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'select',
			array(
				'id'			=> 'fdm-item-flag-icon-size',
				'title'			=> __( 'Menu Item Flag icon size', 'food-and-drink-menu-pro' ),
				'description'	=> __( 'The size in pixels of menu item flag icons (if enabled).', 'food-and-drink-menu-pro' ),
				'options'		=> array(
					'32' => __( '32x32 (default)', 'food-and-drink-menu-pro' ),
					'64' => __( '64x64', 'food-and-drink-menu-pro' )
				)
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'text',
			array(
				'id'			=> 'fdm-styling-section-title-font-family',
				'title'			=> __( 'Section Title Font Family', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the font family for the section titles. (Please note that the font family must already be loaded on the site. This does not load it.)', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'text',
			array(
				'id'			=> 'fdm-styling-section-title-font-size',
				'title'			=> __( 'Section Title Font Size', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the font size for the section titles. Include the unit (e.g. 20px or 2em).', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'colorpicker',
			array(
				'id'			=> 'fdm-styling-section-title-color',
				'title'			=> __( 'Section Title Color', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the color for the section titles', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'text',
			array(
				'id'			=> 'fdm-styling-item-name-font-family',
				'title'			=> __( 'Item Name Font Family', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the font family for the names of the menu items. (Please note that the font family must already be loaded on the site. This does not load it.)', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'text',
			array(
				'id'			=> 'fdm-styling-item-name-font-size',
				'title'			=> __( 'Item Name Font Size', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the font size for the names of the menu items. Include the unit (e.g. 18px or 1.3em).', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'colorpicker',
			array(
				'id'			=> 'fdm-styling-item-name-color',
				'title'			=> __( 'Item Name Color', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the color for the names of the menu items', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'text',
			array(
				'id'			=> 'fdm-styling-item-description-font-family',
				'title'			=> __( 'Item Description Font Family', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the font family for the descriptions of the menu items. (Please note that the font family must already be loaded on the site. This does not load it.)', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'text',
			array(
				'id'			=> 'fdm-styling-item-description-font-size',
				'title'			=> __( 'Item Description Font Size', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the font size for the descriptions of the menu items. Include the unit (e.g. 12px or 1em).', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'colorpicker',
			array(
				'id'			=> 'fdm-styling-item-description-color',
				'title'			=> __( 'Item Description Color', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the color for the descriptions of the menu items', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'text',
			array(
				'id'			=> 'fdm-styling-item-price-font-size',
				'title'			=> __( 'Item Price Font Size', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the font size for the prices of the menu items. Include the unit (e.g. 12px or 1em).', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'colorpicker',
			array(
				'id'			=> 'fdm-styling-item-price-color',
				'title'			=> __( 'Item Price Color', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the color for the prices of the menu items', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'text',
			array(
				'id'			=> 'fdm-styling-image-width',
				'title'			=> __( 'Item Image Width', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the width of the menu item images. Include the unit (e.g. 20% or 200px). Default is 33%.', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'text',
			array(
				'id'			=> 'fdm-styling-image-border-size',
				'title'			=> __( 'Item Image Border Size', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the size of the border around menu item images. It is automatically in pixels, so no need to set the unit (e.g. just put 1 or 3, etc.).', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'colorpicker',
			array(
				'id'			=> 'fdm-styling-image-border-color',
				'title'			=> __( 'Item Image Border Color', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the color for the border around the menu item images', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'text',
			array(
				'id'			=> 'fdm-styling-separating-line-size',
				'title'			=> __( 'Separating Line Size', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the size of the line that separates different menu sections. It is automatically in pixels, so no need to set the unit (e.g. just put 1 or 3, etc.).', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'colorpicker',
			array(
				'id'			=> 'fdm-styling-separating-line-color',
				'title'			=> __( 'Separating Line Color', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the color for the line that separates different menu sections', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'text',
			array(
				'id'			=> 'fdm-styling-filtering-font-family',
				'title'			=> __( 'Filtering Font Family', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the font family for the filtering area. (Please note that the font family must already be loaded on the site. This does not load it.)', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'text',
			array(
				'id'			=> 'fdm-styling-filtering-title-font-size',
				'title'			=> __( 'Filtering Title Font Size', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the font size for the filtering area title. Include the unit (e.g. 20px or 2em).', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'colorpicker',
			array(
				'id'			=> 'fdm-styling-filtering-title-color',
				'title'			=> __( 'Filtering Title Color', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the color for the filtering area title', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'text',
			array(
				'id'			=> 'fdm-styling-filtering-labels-font-size',
				'title'			=> __( 'Filtering Labels Font Size', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the font size for the filtering area labels. Include the unit (e.g. 14px or 1.2em).', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'colorpicker',
			array(
				'id'			=> 'fdm-styling-filtering-labels-color',
				'title'			=> __( 'Filtering Labels Color', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the color for the filtering area labels', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'text',
			array(
				'id'			=> 'fdm-styling-sidebar-font-family',
				'title'			=> __( 'Sidebar Font Family', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the font family for the menu sidebar. (Please note that the font family must already be loaded on the site. This does not load it.)', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'text',
			array(
				'id'			=> 'fdm-styling-sidebar-title-font-size',
				'title'			=> __( 'Sidebar Titles Font Size', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the font size for the section titles in the menu sidebar. Include the unit (e.g. 20px or 2em).', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'colorpicker',
			array(
				'id'			=> 'fdm-styling-sidebar-title-color',
				'title'			=> __( 'Sidebar Titles Color', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the color for the section titles in the menu sidebar', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'text',
			array(
				'id'			=> 'fdm-styling-sidebar-description-font-size',
				'title'			=> __( 'Sidebar Descriptions Font Size', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the font size for the section descriptions in the menu sidebar. Include the unit (e.g. 14px or 1em).', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-styling-settings',
			'colorpicker',
			array(
				'id'			=> 'fdm-styling-sidebar-description-color',
				'title'			=> __( 'Sidebar Descriptions Color', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose the color for the section descriptions in the menu sidebar', 'food-and-drink-menu' )
			)
		);

		// Create a tab for premium settings
		$sap->add_section(
			'food-and-drink-menu-settings',	// Page to add this section to
			array(								// Array of key/value pairs matching the AdminPageSection class constructor variables
				'id'			=> 'fdm-advanced-settings',
				'title'			=> __( 'Advanced', 'food-and-drink-menu' ),
				'is_tab'		=> true
			)
		);

		if ( ! $fdm_controller->permissions->check_permission('filtering') ) {
			$filtering_permissions = array(
				'disabled'		=> true,
				'disabled_image'=> 'https://www.etoilewebdesign.com/wp-content/uploads/2018/06/Logo-White-Filled40-px.png',
				'purchase_link'	=> 'https://www.etoilewebdesign.com/plugins/ultimate-product-catalog/'
			);
		}
		else { $filtering_permissions = array(); }

		// Create a section to enable/disable filtering features
		$sap->add_section(
			'food-and-drink-menu-settings',	// Page to add this section to
			array_merge( 
				array(								// Array of key/value pairs matching the AdminPageSection class constructor variables
					'id'			=> 'fdm-filtering-settings',
					'title'			=> __( 'Filtering', 'food-and-drink-menu' ),
					'description'	=> __( 'Choose what filtering, if any, of the menu items you wish to enable.', 'food-and-drink-menu' ),
					'tab'			=> 'fdm-advanced-settings'
				),
				$filtering_permissions
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-filtering-settings',
			'checkbox',
			array(
				'id'			=> 'fdm-text-search',
				'title'			=> __( 'Menu Item Search', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose what menu items features, if any, should be searchable.', 'food-and-drink-menu' ),
				'options'		=> array(
					'name' 			=> 'Name',
					'description' 	=> 'Description'
				)
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-filtering-settings',
			'toggle',
			array(
				'id'			=> 'fdm-enable-price-filtering',
				'title'			=> __( 'Price Filtering', 'food-and-drink-menu' ),
				'label'			=> __( 'Allow visitors to search menu items in a specific price range.', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-filtering-settings',
			'select',
			array(
				'id'			=> 'fdm-price-filtering-type',
				'title'			=> __( 'Price Filtering Control', 'food-and-drink-menu' ),
				'label'			=> __( 'Choose the type of control available to visitors if price filtering is enabled.', 'food-and-drink-menu' ),
				'blank_option'	=> false,
				'options'		=> array(
					'textbox' 	=> 'Text Boxes',
					'slider' 	=> 'Slider'
				)
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-filtering-settings',
			'toggle',
			array(
				'id'			=> 'fdm-enable-sorting',
				'title'			=> __( 'Menu Item Sorting', 'food-and-drink-menu' ),
				'label'			=> __( 'Allow visitors to sort menu items by name, price, date added, etc. to find items they may be interested in.', 'food-and-drink-menu' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-filtering-settings',
			'checkbox',
			array(
				'id'			=> 'fdm-item-sorting',
				'title'			=> __( 'Sortable Items', 'food-and-drink-menu' ),
				'description'	=> __( 'Choose what menu items features, if any, should be sortable.', 'food-and-drink-menu' ),
				'options'		=> array(
					'name' 			=> 'Name',
					'price' 		=> 'Price',
					'date_added'	=> 'Date Added'
				)
			)
		);

		if ( ! $fdm_controller->permissions->check_permission('flags') ) {
			$flags_permissions = array(
				'disabled'		=> true,
				'disabled_image'=> 'https://www.etoilewebdesign.com/wp-content/uploads/2018/06/Logo-White-Filled40-px.png',
				'purchase_link'	=> 'https://www.etoilewebdesign.com/plugins/ultimate-product-catalog/'
			);
		}
		else { $flags_permissions = array(); }
		
		$sap->add_section(
			'food-and-drink-menu-settings',	// Page to add this section to
			array_merge( 
				array(								// Array of key/value pairs matching the AdminPageSection class constructor variables
					'id'			=> 'fdm-advanced-enable-settings',
					'title'			=> __( 'Functionality', 'food-and-drink-menu' ),
					'description'	=> __( 'Choose what features of the menu items you wish to enable or disable.', 'food-and-drink-menu' ),
					'tab'			=> 'fdm-advanced-settings'
				),
				$flags_permissions
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-advanced-enable-settings',
			'radio',
			array(
				'id'			=> 'fdm-related-items',
				'title'			=> __( 'Related Items', 'food-and-drink-menu' ),
				'label'			=> __( 'Should related items be displayed when viewing a particular item.', 'food-and-drink-menu' ),
				'options'		=> array(
					'none'			=> 'None',
					'automatic'		=> 'Automatic',
					'manual'		=> 'Manual'
				)
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-advanced-enable-settings',
			'toggle',
			array(
				'id'			=> 'fdm-disable-menu-item-flags',
				'title'			=> __( 'Disable Menu Item Flags', 'food-and-drink-menu-pro' ),
				'label'			=> __( 'Disable the flags which can be assigned to menu items.', 'food-and-drink-menu-pro' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-advanced-enable-settings',
			'toggle',
			array(
				'id'			=> 'fdm-disable-specials',
				'title'			=> __( 'Disable Menu Item Specials', 'food-and-drink-menu-pro' ),
				'label'			=> __( 'Disable the specials options for menu items.', 'food-and-drink-menu-pro' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-advanced-enable-settings',
			'toggle',
			array(
				'id'			=> 'fdm-disable-price-discounted',
				'title'			=> __( 'Disable Discounted Price', 'food-and-drink-menu-pro' ),
				'label'			=> __( 'Disable discounted pricing options for menu items.', 'food-and-drink-menu-pro' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-advanced-enable-settings',
			'toggle',
			array(
				'id'			=> 'fdm-disable-src',
				'title'			=> __( 'Disable Source', 'food-and-drink-menu-pro' ),
				'label'			=> __( 'Disable all source options in menus.', 'food-and-drink-menu-pro' )
			)
		);

		// Adds in a section to handle the source map options
		$sap->add_section(
			'food-and-drink-menu-settings',
			array(
				'id'    => 'fdm-google-map',
				'title' => __( 'Google Map', 'food-and-drink-menu-pro' ),
				'tab'	=> 'fdm-advanced-settings'
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-google-map',
			'toggle',
			array(
				'id'			=> 'fdm-disable-src-map',
				'title'			=> __( 'Disable Source Map', 'food-and-drink-menu-pro' ),
				'label'			=> __( 'Disable the source map.', 'food-and-drink-menu-pro' )
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-google-map',
			'text',
			array(
				'id'          => 'fdm-google-map-api-key',
				'title'       => __( 'Google Maps API Key', 'food-and-drink-menu-pro' ),
				'description' => sprintf(
					__( 'Google requires an API key to use their maps. %sGet an API key%s. A full walk-through is available in the %sdocumentiaton%s.', 'food-and-drink-menu-pro' ),
					'<a href="https://developers.google.com/maps/documentation/javascript/get-api-key">',
					'</a>',
					'<a href="http://doc.themeofthecrop.com/plugins/food-and-drink-menu/user/pro/google-map-api-key">',
					'</a>'
				),
			)
		);

		if ( ! $fdm_controller->permissions->check_permission('custom_fields') ) {
			$custom_field_permissions = array(
				'disabled'		=> true,
				'disabled_image'=> 'https://www.etoilewebdesign.com/wp-content/uploads/2018/06/Logo-White-Filled40-px.png',
				'purchase_link'	=> 'https://www.etoilewebdesign.com/plugins/ultimate-product-catalog/'
			);
		}
		else { $custom_field_permissions = array(); }


		// Create a tab for custom field settings
		$sap->add_section(
			'food-and-drink-menu-settings',	// Page to add this section to
			array(								// Array of key/value pairs matching the AdminPageSection class constructor variables
				'id'			=> 'fdm-custom-fields-settings',
				'title'			=> __( 'Custom Fields', 'food-and-drink-menu' ),
				'is_tab'		=> true
			)
		);

		// Create a section for the custom fields
		$sap->add_section(
			'food-and-drink-menu-settings',	// Page to add this section to
			array_merge( 
				array(								// Array of key/value pairs matching the AdminPageSection class constructor variables
					'id'			=> 'fdm-custom-fields-settings-section',
					'title'			=> __( 'Custom Fields', 'food-and-drink-menu' ),
					'description'	=> __( 'Create custom fields for your menu.', 'food-and-drink-menu' ),
					'tab'			=> 'fdm-custom-fields-settings',
				),
				$custom_field_permissions
			)
		);
		$sap->add_setting(
			'food-and-drink-menu-settings',
			'fdm-custom-fields-settings-section',
			'infinite_table',
			array(
				'id'			=> 'fdm-custom-fields',
				'title'			=> __( 'Custom Fields', 'food-and-drink-menu' ),
				'add_label'		=> __( 'Add Field', 'food-and-drink-menu' ),
				'description'	=> __( 'Use this table to create custom fields that can be filled in for each menu item. This information is displayed when a specific item is viewed, either on it\'s own page or via the lightbox if you have that option enabled.<br /> Looking to add nutritional information? <span class="fdm-custom-fields-add-nutrional-information">Add it in one click</span>.', 'food-and-drink-menu' ),
				'fields'		=> array(
					'name' => array(
						'type' 		=> 'text',
						'label' 	=> 'Name',
						'required' 	=> true
					),
					'slug' => array(
						'type' 		=> 'text',
						'label' 	=> 'Slug',
						'required' 	=> true
					),
					'type' => array(
						'type' 		=> 'select',
						'label'		=> 'Type',
						'required'	=> true,
						'options' 	=> array(
							'section' 	=> 'Section',
							'text' 		=> 'Short Text',
							'textarea' 	=> 'Long Text',
							'select'	=> 'Dropdown',
							'checkbox'	=> 'Checkboxes'
						)
					),
					'values' => array(
						'type'		=> 'text',
						'label'		=> 'Values',
						'required'	=> false
					)
				)
			)
		);

		// Create filter so addons can modify the settings page or add new pages
		$sap = apply_filters( 'fdm_settings_page', $sap );

		// Backwards compatibility when the sap library went to version 2
		$sap->port_data(2);

		// Register all admin pages and settings with WordPress
		$sap->add_admin_menus();
	}

	/**
	 * Set the style of a menu or menu item before rendering
	 * @since 1.1
	 */
	public function set_style( $args ) {
		global $fdm_controller;

		if ( !$fdm_controller->settings->get_setting('fdm-style') ) {
			$args['style'] = 'base';
		} else {
			$args['style'] = $fdm_controller->settings->get_setting('fdm-style');
		}

		return $args;
	}

}
